<#
	.SYNOPSIS
		Query your PRTG Instance for Maps with missing objects (Maps were the corresponding objects have been deleted and the text "Object [OBJID] does not exist")

	.DESCRIPTION
        This script is able to check all PRTG maps (Dashboards) for missing objects(Maps were the corresponding objects have been deleted and the text "Object [OBJID] does not exist"). Credentials are needed to retrieve the map list and each map itself. The script is designed to be used with PRTG Network Monitor (https://www.paessler.com/prtg). IMPORTANT: Remember to configure the appropriate limits in the channel settings for the expiration to get alerts.

	.PARAMETER Uri
		The Uniform Resource Identifier (URI) of the PRTG Server, e.g.: https://prtg.example.org or http://myprtgserver.example.com (Important: Without a trailing slash (/))

	.PARAMETER UserName
		The login name of the PRTG account used for the query(to retrieve the maps), e.g.: svc-api-mapstester. If the user doesn't see a map, it won't be inspected

	.PARAMETER Passhash
		The user's passhash (Visible under Setup > User Accounts in PRTG's Interface

	.PARAMETER Timeout
		The number of seconds to wait for EACH HTTP query to complete (Default: 10). This can quickly add to a lot, for instance if you have 100 maps @ 10 seconds each that's 1000 seconds (16min) as each map needs to be queried individually

	.PARAMETER TrustAllCerts
		Switch parameter to force Powershell to connect to endpoints with SSL Issues (Self-Signed cert, wrong cert, etc) which introduces security risks (Default: Disabled)

	.EXAMPLE
		Running the script interactively to test it:
		PS> .\Get-PRTGMapsBroken.ps1 -Uri https://prtg.example.org -UserName svc-api-mapstester -Passhash 1234567896 -Timeout 5

	.EXAMPLE
		For running the script in PRTG, use PRTG's placeholders to make the setup more robust (and optionally ignore SSL Certificate Errors with -TrustAllCerts)
		Parameters(in the Sensor's settings): -Uri 'https://%host' -UserName '%windowsuser' -Passhash '%windowspassword' -Timeout 5 -TrustAllCerts

	.NOTES
		Version: 0.13.20220427
		Author: Luciano Lingnau, Paessler AG (https://www.paessler.com/)

	.LINK
		https://kb.paessler.com/en/topic/90645
#>

#Requires -Version 3.0

param(
    [Parameter(Mandatory=$true)]
	[string]$Uri,
	[Parameter(Mandatory=$true)]
	[string]$UserName,
	[Parameter(Mandatory=$true)]
	[string]$Passhash,
	[int]$Timeout = "10",
	[switch]$TrustAllCerts = $false
)

[string]$sensorMessage = ''
$errorFlag = $false
$results = @()

#Avoid issues with weird SSL Certificates (Self-signed, expired, etc)
#Should only be used when a trusted certificate is not an option
if ($TrustAllCerts){
add-type @"
    using System.Net;
    using System.Security.Cryptography.X509Certificates;
    public class TrustAllCertsPolicy : ICertificatePolicy {
        public bool CheckValidationResult(
            ServicePoint srvPoint, X509Certificate certificate,
            WebRequest request, int certificateProblem) {
            return true;
        }
    }
"@
[System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy
}

#Force TLS1.1 and TLS1.2 otherwise the script might fail in some systems/Powershell Versions
[Net.ServicePointManager]::SecurityProtocol = "Tls11,Tls12"

try {

	#Stopwatch used to time the lenghtiest part of the execution (long execution times could be an issue)
	$exectime = [System.Diagnostics.Stopwatch]::StartNew()

    #Get all maps so we can iterate over every map ID
    $b = Invoke-WebRequest -Uri "$($Uri)/api/table.xml?content=maps&output=xml&columns=objid,name&username=$($UserName)&passhash=$($Passhash)" -TimeoutSec $Timeout -UseBasicParsing -ErrorAction SilentlyContinue
    $xml = [xml]$b.Content

    #This is the node that contains the info we need (objid & name)
    $nodes = $xml.SelectNodes("//item")

    #Store everything in an object to simplify the query later
    $objs = @()

    foreach ($node in $nodes) {

        #Query the map using the username and password
        $a = Invoke-WebRequest -Uri "$($Uri)/controls/maponly.htm?id=$($node.objid)&username=$($UserName)&passhash=$($Passhash)" -TimeoutSec $Timeout -UseBasicParsing -ErrorAction SilentlyContinue

        #Count the number of missing objects on each map
        $missingCount = ($a.Content | Select-String -Pattern "does not exist" -AllMatches).Matches.Count
        
        #Store everything in an object to simplify the query later
        #$obj = New-Object PSObject -prop @{objid=$node.objid;name=$node.name;secretKey=$xmlSecretkey.prtg.result;missingCount=$missingCount}
        $obj = New-Object PSObject -prop @{objid=$node.objid;name=$node.name;missingCount=$missingCount}
        $objs += $obj

    }

    #Stop Stopwatch used to time the lenghtiest part of the execution
    $exectime.Stop()
	
	#Parse Results in a PRTG-Friendly fashion
	$results += @{
		Channel = 'Maps Broken'
		Value = $objs | Where-Object {$_.missingCount -gt 0} | Measure-Object | Select-Object -ExpandProperty Count
		Unit = 'Count'
	}
	$results += @{
		Channel = 'Map-Objects Missing'
		Value = $objs | Measure-Object -Property missingCount -Sum | Select-Object -ExpandProperty Sum
		Unit = 'Count'
	}
	$results += @{
		Channel = 'Maps Total'
		Value = $objs.Count
		Unit = 'Count'
	}
	$results += @{
		Channel = 'Execution Time'
		Value = $exectime.ElapsedMilliseconds
		Unit = 'TimeResponse'
	}

	$sensorMessage = ""
	
	#Sensor message only needs to be generated when there's at least 1 map with missing objects
    if (($objs | Where-Object {$_.missingCount -gt 0} | Measure-Object | Select-Object -ExpandProperty Count) -gt 0) {
		$sensorMessage = "Top affected map(s): "
		$objs | Where-Object {$_.missingCount -gt 0} | Sort-Object -Property missingCount -Descending | Select-Object -First 5 | ForEach {
			 $sensorMessage += "($($_.objid))$($_.name); "
		}
	}
	
	#Remove trailing space (Cosmetic)
    $sensorMessage = $sensorMessage.TrimEnd()
	#Remove Special Characters that could break the PRTG Parsing. Credits to bjornsallarp(https://gist.github.com/bjornsallarp/3304522)
	$sensorMessage = [Text.Encoding]::ASCII.GetString([Text.Encoding]::GetEncoding("Cyrillic").GetBytes($sensorMessage))

} catch {
	$errorFlag  = $true
	$sensorMessage = "Exception (Line $($_.InvocationInfo.ScriptLineNumber)): $($_.Exception.Message)"
}

$sensorResult = '' | Select-Object prtg
if ($errorFlag) {
	$sensorResult.prtg = '' | Select-Object error
	$sensorResult.prtg.error = 1
} else {
	$sensorResult.prtg = '' | Select-Object result
	$sensorResult.prtg.result = $results
}
if ($sensorMessage) {
	$sensorResult.prtg | Add-Member -MemberType NoteProperty -Name 'text' -Value $null
	$sensorResult.prtg.text = $sensorMessage
}
$sensorResultJson = $sensorResult | ConvertTo-Json -Depth 5
Write-Output $sensorResultJson